// Show Failed URL 0.1.4
// By Pike

// Contributor(s):
// Nickolay Ponomarev (Reload loads failed page instead of error page)

var ShowFailedURL = {

	loaded: false,
	
	onLoad: function()
	{
		// Overwrite original functions with modified versions if not already done
		if (!this.loaded)
		{
			nsBrowserStatusHandler.prototype.origOnLocationChange = nsBrowserStatusHandler.prototype.onLocationChange;
			nsBrowserStatusHandler.prototype.onLocationChange = this.onLocationChange;
			handleURLBarRevert = this.handleURLBarRevert;
			this.origBrowserReloadWithFlags = BrowserReloadWithFlags;
			BrowserReloadWithFlags = this.BrowserReloadWithFlags;
			this.loaded = true;
		}
	},
	
	isErrorPage: function(url)
	{
		return /^chrome:.*\/netError\.xhtml/.test(url);
	},
	
	getFailedUrl: function(url)
	{
		var urlStart = url.search(/u\=/);
		var urlEnd = url.search(/\&d\=/);
		var urlSlice = decodeURIComponent(url.slice(urlStart + 2, urlEnd));
		var failedUrl = null;
		try
		{
			var utf8ConverterService = Components.classes['@mozilla.org/intl/utf8converterservice;1'].getService(Components.interfaces.nsIUTF8ConverterService);
			failedUrl = utf8ConverterService.convertStringToUTF8(urlSlice, null, false);
		}
		catch (err)
		{
			failedUrl = urlSlice;
		}
		return failedUrl;
	},
	
	onLocationChange: function(aWebProgress, aRequest, aLocation)
	// Modified version of nsBrowserStatusHandler.prototype.onLocationChange() from browser.js
	{
		this.origOnLocationChange(aWebProgress, aRequest, aLocation);
		
		var changeUrlBar = true;
		if (typeof getBrowser().selectedBrowser.userTypedValue != 'undefined')
		{
			changeUrlBar = (getBrowser().selectedBrowser.userTypedValue == null);
		}
		if (changeUrlBar)
		{
			var location = aLocation.spec;
			if (ShowFailedURL.isErrorPage(location))
			{
				// Change URL bar
				location = ShowFailedURL.getFailedUrl(location);
				setTimeout(function(url) {gURLBar.value = url;}, 0, location);
				
				// Select URL bar
				setTimeout(function() {gURLBar.select();}, 100);
			}
		}
	},
	
	handleURLBarRevert: function()
	// Modified version of handleURLBarRevert() from browser.js
	{
		var url = getWebNavigation().currentURI.spec;
		var throbberElement = document.getElementById('navigator-throbber');
		var isScrolling = gURLBar.popupOpen;
		if ((!throbberElement || !throbberElement.hasAttribute('busy')) && !isScrolling)
		{
			if (ShowFailedURL.isErrorPage(url))
			{
				gURLBar.value = ShowFailedURL.getFailedUrl(url);
			}
			else if (url != 'about:blank')
			{
				gURLBar.value = url;
				gURLBar.select();
				SetPageProxyState('valid', null);
			}
			else
			{
				gURLBar.value = '';
			}
		}
		if (typeof gBrowser.userTypedValue != 'undefined') gBrowser.userTypedValue = null;
		return !isScrolling;
	},
	
	BrowserReloadWithFlags: function(reloadFlags)
	// Replaces BrowserReloadWithFlags() from browser.js
	{
		var url = getWebNavigation().currentURI.spec;
		if (ShowFailedURL.isErrorPage(url))
		{
			// Reload failed page instead of error page
			loadURI(ShowFailedURL.getFailedUrl(url));
		}
		else
		{
			ShowFailedURL.origBrowserReloadWithFlags(reloadFlags);
		}
	}
};

if (local_common.IsOldBrowser()) {
  window.addEventListener('load', function() {ShowFailedURL.onLoad();}, false);
}
